import choco.*;
import choco.integer.*;
import choco.integer.IntDomainVar;
import choco.Problem;

public class Choco{
	
	public static String[] camminoDurataMassimo(boolean[][] matriceAd,int[] durate){ //l'array durate indica le durate delle attivita
		int[][] grafo = tabellaGrafo(matriceAd);									 //bisogna costruire l'array delle durate in base
		int[] durateArchi = durateArchi(durate,grafo);  // ---------------------->    agli archi
		int n = grafo.length;

		for(int i = 0; i < grafo.length; i++){
			System.out.println(grafo[i][0] + " " + grafo[i][1]);
		}
		Problem p = new Problem();
		
		//variabile V contiene al suo interno le informazioni sugli archi,
		//se la cella di indice i e > 1 --> significa che l'arco corrispondente
		//fara parte del cammino ed il valore ci dice in che ordine l'arco e visitato dal cammino
		//la corrispondenza indice del vettore archi e data dalla tabella grafo
		//In particolare abbiamo che l'indice 0 del vettore v rappresenta l'arco presente 
		//nella riga 0 della tabella grafo
		
		// v[0] --> grafo[0]
		// v[1] --> grafo[1]
		// v[2] --> grafo[2]
		// v[3] --> grafo[3]
		// v[4] --> grafo[4]
		// v[5] --> grafo[5]
		
		
		
		IntDomainVar[] v = new IntDomainVar[n];

		for(int i =0; i < n; i++){
			v[i] = p.makeEnumIntVar("arco"+i,0,n);
		}
		
				
		/**************************************************************
		 *La variabile durata che choco dovra massimizzare
		 */
		 
		IntDomainVar durata = p.makeEnumIntVar("durata",0,100000);
		
		/*****************************************************************
		 *Array booleano della durate --> se nella posizione i dell'array
		 *c' il valore 1 --> bisogna considerare la durata di quel arco
		 */
		
		
		IntDomainVar[] bool = new IntDomainVar[n];
		for(int i = 0; i < n; i++){
			bool[i] = p.makeEnumIntVar("bool",0,1);
			p.post(p.implies(p.eq(0,v[i]),p.eq(bool[i],0)));
			p.post(p.implies(p.eq(bool[i],0),p.eq(v[i],0)));
		}
		
		/************************************************
		 *vincolo che lega l'array boolean e l'array delle durate con 
		 *la variabile durata che sara oggetto dela funzione obiettivo
		 */
		p.post(p.eq(p.scalar(durateArchi,bool),durata));
		
		 
		//********************************************/
		//vincolo nodo di partenza
		
		IntDomainVar[] archiIniziali = archiIniziali(v,grafo);
		
		p.post(p.eq(p.sum(archiIniziali),1));

		//********************************************/
		//vincolo nodo di arrivo
		
		IntDomainVar[] archiFinali = archiFinale(v,grafo,matriceAd);
		
		p.post(p.leq(1,p.sum(archiFinali)));
		
		//Vincolo somma successori
		//Se un arco i  maggiore di zero --> la somma dei successori  uguale a i+1
		
		for(int i = 0; i < n ; i ++){
			IntDomainVar[] succ = successori(v,i,grafo);
			if(succ!=null){
				p.post(p.implies(p.lt(0,v[i]),p.eq(p.sum(succ),p.plus(v[i],1))));
			}
		}
		
		//Vincolo somma predecessori
		//Se un arco i  maggiore di zero --> la somma dei predecessori  uguale a i-1
		
		for(int i = 0; i < n ; i ++){
			IntDomainVar[] pred = predecessori(v,i,grafo);
			if(pred!=null){
				p.post(p.implies(p.lt(0,v[i]),p.eq(p.sum(pred),p.minus(v[i],1))));
			}
		}
		//vincolo predecessori e successori
		
		
		for(int i = 0; i < n ;i++){
			for(int j = 0; j <n ; j++){
				if(grafo[i][1] == grafo[j][0]){
					p.post(p.or(p.eq(v[j],0),p.eq(v[i],0),p.eq(v[j],p.plus(v[i],1))));
				}
			}
		}
		//successori
		
		
		for(int i = 0; i < n ; i ++){
			IntDomainVar[] succ = successori(v,i,grafo);
			if(succ!=null){
				p.post(p.implies(p.eq(p.sum(succ),0),p.eq(v[i],0)));
			}
		}
		
		//predecessori
		
		
		for(int i = 0; i < n ; i ++){
			IntDomainVar[] pred = predecessori(v,i,grafo);
			if(pred!=null){
				p.post(p.implies(p.eq(p.sum(pred),0),p.eq(v[i],0)));
			}
			
		}
		
		for(int i = archiIniziali.length; i < n ; i ++){
			IntDomainVar[] pred = predecessori(v,i,grafo);
			if(pred!=null){
				p.post(p.implies(p.lt(0,v[i]),p.eq(p.sum(pred),p.minus(v[i],1))));
			}else{
				try{
					IntDomainVar[] succ = successori(v,i,grafo);
					v[i].setVal(0);
					if(succ!=null)
						p.post(p.eq(p.sum(succ),0));
				}catch(Exception ex){
				}
			}
		}
		
		
		/*******************************************************
		 *Funzione obiettivo
		 */
		 	
		p.maximize(durata,false);
		
		Solver s = p.getSolver();
    	p.solve(true);
    	
    	int soluzioniTrovate = s.getSearchSolver().solutions.size();
    	
    	if(soluzioniTrovate>0){
    		String[] soluzioni = new String[n+1];
    		Solution solution = (Solution) s.getSearchSolver().solutions.get(0);
     			
 			for (int j = 0; j < n+1; j++) {
     			soluzioni[j] = ""+solution.getValue(j);
 			}
 			return soluzioni;
    	}else{
    		return null;
    	}
	}
	
	public static String[] camminoDurataMinimo(boolean[][] matriceAd,int[] durate){ //l'array durate indica le durate delle attivita
		int[][] grafo = tabellaGrafo(matriceAd);									 //bisogna costruire l'array delle durate in base
		int[] durateArchi = durateArchi(durate,grafo);  // ---------------------------->    agli archi
		int n = grafo.length;

		Problem p = new Problem();
		
		//variabile V contiene al suo interno le informazioni sugli archi,
		//se la cella di indice i e > 1 --> significa che l'arco corrispondente
		//fara parte del cammino ed il valore ci dice in che ordine l'arco e visitato dal cammino
		//la corrispondenza indice del vettore archi e data dalla tabella grafo
		//In particolare abbiamo che l'indice 0 del vettore v rappresenta l'arco presente 
		//nella riga 0 della tabella grafo
		
		// v[0] --> grafo[0]
		// v[1] --> grafo[1]
		// v[2] --> grafo[2]
		// v[3] --> grafo[3]
		// v[4] --> grafo[4]
		// v[5] --> grafo[5]
		
		
		
		IntDomainVar[] v = new IntDomainVar[n];

		for(int i =0; i < n; i++){
			v[i] = p.makeEnumIntVar("arco"+i,0,n);
		}
		
				
		/**************************************************************
		 *La variabile durata che choco dovra minimizzare
		 */
		 
		IntDomainVar durata = p.makeEnumIntVar("durata",0,100000);
		
		/*****************************************************************
		 *Array booleano della durate --> se nella posizione i dell'array
		 *c' il valore 1 --> bisogna considerare la durata di quel arco
		 */
		
		
		IntDomainVar[] bool = new IntDomainVar[n];
		for(int i = 0; i < n; i++){
			bool[i] = p.makeEnumIntVar("bool",0,1);
			p.post(p.implies(p.eq(0,v[i]),p.eq(bool[i],0)));
			p.post(p.implies(p.eq(bool[i],0),p.eq(v[i],0)));
		}
		
		/************************************************
		 *vincolo che lega l'array boolean e l'array delle durate con 
		 *la variabile durata che sara oggetto dela funzione obiettivo
		 */
		p.post(p.eq(p.scalar(durateArchi,bool),durata));
		
		 
		//********************************************/
		//vincolo nodo di partenza
		
		IntDomainVar[] archiIniziali = archiIniziali(v,grafo);
		
		p.post(p.eq(p.sum(archiIniziali),1));
		
		//********************************************/
		//vincolo nodo di arrivo
		
		IntDomainVar[] archiFinali = archiFinale(v,grafo,matriceAd);
		
		p.post(p.leq(1,p.sum(archiFinali)));
		
		//Vincolo somma successori
		//Se un arco  maggiore di zero --> la somma dei successori  uguale a i+1
		
		for(int i = 0; i < n ; i ++){
			IntDomainVar[] succ = successori(v,i,grafo);
			if(succ!=null){
				p.post(p.implies(p.lt(0,v[i]),p.eq(p.sum(succ),p.plus(v[i],1))));
			}
		}
		
		//Vincolo somma predecessori
		//Se un arco  maggiore di zero --> la somma dei predecessori  uguale a i-1
		
		for(int i = 0; i < n ; i ++){
			IntDomainVar[] pred = predecessori(v,i,grafo);
			if(pred!=null){
				p.post(p.implies(p.lt(0,v[i]),p.eq(p.sum(pred),p.minus(v[i],1))));
			}
		}
		//vincolo predecessori e successori
		
		
		for(int i = 0; i < n ;i++){
			for(int j = 0; j <n ; j++){
				if(grafo[i][1] == grafo[j][0]){
					p.post(p.or(p.eq(v[j],0),p.eq(v[i],0),p.eq(v[j],p.plus(v[i],1))));
				}
			}
		}
		//successori
		
		
		for(int i = 0; i < n ; i ++){
			IntDomainVar[] succ = successori(v,i,grafo);
			if(succ!=null){
				p.post(p.implies(p.eq(p.sum(succ),0),p.eq(v[i],0)));
			}
		}
		
		//predecessori
		
		
		for(int i = 0; i < n ; i ++){
			IntDomainVar[] pred = predecessori(v,i,grafo);
			if(pred!=null){
				p.post(p.implies(p.eq(p.sum(pred),0),p.eq(v[i],0)));
			}
			
		}
		
		for(int i = archiIniziali.length; i < n ; i ++){
			IntDomainVar[] pred = predecessori(v,i,grafo);
			if(pred!=null){
				p.post(p.implies(p.lt(0,v[i]),p.eq(p.sum(pred),p.minus(v[i],1))));
			}else{
				try{
					IntDomainVar[] succ = successori(v,i,grafo);
					v[i].setVal(0);
					if(succ!=null)
						p.post(p.eq(p.sum(succ),0));
				}catch(Exception ex){
				}
			}
		}
		/*******************************************************
		 *Funzione obiettivo
		 */
		 	
		p.minimize(durata,false);
		
		Solver s = p.getSolver();
    	p.solve(true);
    	
    	int soluzioniTrovate = s.getSearchSolver().solutions.size();
    	
    	if(soluzioniTrovate>0){
    		String[] soluzioni = new String[n+1];
    		Solution solution = (Solution) s.getSearchSolver().solutions.get(0);
     			
 			for (int j = 0; j < n+1; j++) {
     			soluzioni[j] = ""+solution.getValue(j);
 			}
 			return soluzioni;
    	}else{
    		return null;
    	}
	}
	
	public static String[] camminoCostoMassimo(boolean[][] matriceAd,int[] costi){ //l'array durate indica le durate delle attivita
		int[][] grafo = tabellaGrafo(matriceAd);									 //bisogna costruire l'array delle durate in base
		int[] costoArchi = costoArchi(costi,grafo);  // ---------------------------->    agli archi
		int n = grafo.length;

		Problem p = new Problem();
		
		//variabile V contiene al suo interno le informazioni sugli archi,
		//se la cella di indice i e > 1 --> significa che l'arco corrispondente
		//fara parte del cammino ed il valore ci dice in che ordine l'arco e visitato dal cammino
		//la corrispondenza indice del vettore archi e data dalla tabella grafo
		//In particolare abbiamo che l'indice 0 del vettore v rappresenta l'arco presente 
		//nella riga 0 della tabella grafo
		
		// v[0] --> grafo[0]
		// v[1] --> grafo[1]
		// v[2] --> grafo[2]
		// v[3] --> grafo[3]
		// v[4] --> grafo[4]
		// v[5] --> grafo[5]
		
		
		
		IntDomainVar[] v = new IntDomainVar[n];

		for(int i =0; i < n; i++){
			v[i] = p.makeEnumIntVar("arco"+i,0,n);
		}
		
				
		/**************************************************************
		 *La variabile durata che choco dovra massimizzare
		 */
		 
		IntDomainVar costo = p.makeEnumIntVar("costo",0,100000);
		
		/*****************************************************************
		 *Array booleano della durate --> se nella posizione i dell'array
		 *c' il valore 1 --> bisogna considerare la durata di quel arco
		 */
		
		
		IntDomainVar[] bool = new IntDomainVar[n];
		for(int i = 0; i < n; i++){
			bool[i] = p.makeEnumIntVar("bool",0,1);
			p.post(p.implies(p.eq(0,v[i]),p.eq(bool[i],0)));
			p.post(p.implies(p.eq(bool[i],0),p.eq(v[i],0)));
		}
		
		/************************************************
		 *vincolo che lega l'array boolean e l'array delle durate con 
		 *la variabile durata che sara oggetto dela funzione obiettivo
		 */
		p.post(p.eq(p.scalar(costoArchi,bool),costo));
		
		 
		//********************************************/
		//vincolo nodo di partenza
		
		IntDomainVar[] archiIniziali = archiIniziali(v,grafo);
		
		p.post(p.eq(p.sum(archiIniziali),1));

	
		//********************************************/
		//vincolo nodo di arrivo
		
		IntDomainVar[] archiFinali = archiFinale(v,grafo,matriceAd);
		
		p.post(p.leq(1,p.sum(archiFinali)));
		
		//Vincolo somma successori
		//Se un arco  maggiore di zero --> la somma dei successori  uguale a i+1
		
		for(int i = 0; i < n ; i ++){
			IntDomainVar[] succ = successori(v,i,grafo);
			if(succ!=null){
				p.post(p.implies(p.lt(0,v[i]),p.eq(p.sum(succ),p.plus(v[i],1))));
			}
		}
		
		//Vincolo somma predecessori
		//Se un arco  maggiore di zero --> la somma dei predecessori  uguale a i-1
		
		for(int i = 0; i < n ; i ++){
			IntDomainVar[] pred = predecessori(v,i,grafo);
			if(pred!=null){
				p.post(p.implies(p.lt(0,v[i]),p.eq(p.sum(pred),p.minus(v[i],1))));
			}
		}
		//vincolo predecessori e successori
		
		
		for(int i = 0; i < n ;i++){
			for(int j = 0; j <n ; j++){
				if(grafo[i][1] == grafo[j][0]){
					p.post(p.or(p.eq(v[j],0),p.eq(v[i],0),p.eq(v[j],p.plus(v[i],1))));
				}
			}
		}
		//successori
		
		
		for(int i = 0; i < n ; i ++){
			IntDomainVar[] succ = successori(v,i,grafo);
			if(succ!=null){
				p.post(p.implies(p.eq(p.sum(succ),0),p.eq(v[i],0)));
			}
		}
		
		//predecessori
		
		
		for(int i = 0; i < n ; i ++){
			IntDomainVar[] pred = predecessori(v,i,grafo);
			if(pred!=null){
				p.post(p.implies(p.eq(p.sum(pred),0),p.eq(v[i],0)));
			}
			
		}
			
		for(int i = archiIniziali.length; i < n ; i ++){
			IntDomainVar[] pred = predecessori(v,i,grafo);
			if(pred!=null){
				p.post(p.implies(p.lt(0,v[i]),p.eq(p.sum(pred),p.minus(v[i],1))));
			}else{
				try{
					IntDomainVar[] succ = successori(v,i,grafo);
					v[i].setVal(0);
					if(succ!=null)
						p.post(p.eq(p.sum(succ),0));
				}catch(Exception ex){
				}
			}
		}		
		
		/*******************************************************
		 *Funzione obiettivo
		 */
		 	
		p.maximize(costo,false);
		
		Solver s = p.getSolver();
    	p.solve(true);
    	
    	int soluzioniTrovate = s.getSearchSolver().solutions.size();
    	
    	if(soluzioniTrovate>0){
    		String[] soluzioni = new String[n+1];
    		Solution solution = (Solution) s.getSearchSolver().solutions.get(0);
     			
 			for (int j = 0; j < n+1; j++) {
     			soluzioni[j] = ""+solution.getValue(j);
 			}
 			return soluzioni;
    	}else{
    		return null;
    	}
	}
	
	public static String[] camminoCostoMinimo(boolean[][] matriceAd,int[] costi){ //l'array durate indica le durate delle attivita
		int[][] grafo = tabellaGrafo(matriceAd);									 //bisogna costruire l'array delle durate in base
		int[] costoArchi = costoArchi(costi,grafo);  // ---------------------------->    agli archi
		int n = grafo.length;

		Problem p = new Problem();
		
		//variabile V contiene al suo interno le informazioni sugli archi,
		//se la cella di indice i e > 1 --> significa che l'arco corrispondente
		//fara parte del cammino ed il valore ci dice in che ordine l'arco e visitato dal cammino
		//la corrispondenza indice del vettore archi e data dalla tabella grafo
		//In particolare abbiamo che l'indice 0 del vettore v rappresenta l'arco presente 
		//nella riga 0 della tabella grafo
		
		// v[0] --> grafo[0]
		// v[1] --> grafo[1]
		// v[2] --> grafo[2]
		// v[3] --> grafo[3]
		// v[4] --> grafo[4]
		// v[5] --> grafo[5]
		
		
		
		IntDomainVar[] v = new IntDomainVar[n];

		for(int i =0; i < n; i++){
			v[i] = p.makeEnumIntVar("arco"+i,0,n);
		}
		
				
		/**************************************************************
		 *La variabile durata che choco dovra massimizzare
		 */
		 
		IntDomainVar costo = p.makeEnumIntVar("costo",0,100000);
		
		/*****************************************************************
		 *Array booleano della durate --> se nella posizione i dell'array
		 *c' il valore 1 --> bisogna considerare la durata di quel arco
		 */
		
		
		IntDomainVar[] bool = new IntDomainVar[n];
		for(int i = 0; i < n; i++){
			bool[i] = p.makeEnumIntVar("bool",0,1);
			p.post(p.implies(p.eq(0,v[i]),p.eq(bool[i],0)));
			p.post(p.implies(p.eq(bool[i],0),p.eq(v[i],0)));
		}
		
		/************************************************
		 *vincolo che lega l'array boolean e l'array delle durate con 
		 *la variabile durata che sara oggetto dela funzione obiettivo
		 */
		p.post(p.eq(p.scalar(costoArchi,bool),costo));
		
		 
		//********************************************/
		//vincolo nodo di partenza
		
		IntDomainVar[] archiIniziali = archiIniziali(v,grafo);
		
		p.post(p.eq(p.sum(archiIniziali),1));


		//********************************************/
		//vincolo nodo di arrivo
		
		IntDomainVar[] archiFinali = archiFinale(v,grafo,matriceAd);
		
		p.post(p.leq(1,p.sum(archiFinali)));
				
		//Vincolo somma successori
		//Se un arco  maggiore di zero --> la somma dei successori  uguale a i+1
		
		for(int i = 0; i < n ; i ++){
			IntDomainVar[] succ = successori(v,i,grafo);
			if(succ!=null){
				p.post(p.implies(p.lt(0,v[i]),p.eq(p.sum(succ),p.plus(v[i],1))));
			}
		}
		
		//Vincolo somma predecessori
		//Se un arco  maggiore di zero --> la somma dei predecessori  uguale a i-1
		
		for(int i = 0; i < n ; i ++){
			IntDomainVar[] pred = predecessori(v,i,grafo);
			if(pred!=null){
				p.post(p.implies(p.lt(0,v[i]),p.eq(p.sum(pred),p.minus(v[i],1))));
			}
		}
		//vincolo predecessori e successori
		
		
		for(int i = 0; i < n ;i++){
			for(int j = 0; j <n ; j++){
				if(grafo[i][1] == grafo[j][0]){
					p.post(p.or(p.eq(v[j],0),p.eq(v[i],0),p.eq(v[j],p.plus(v[i],1))));
				}
			}
		}
		//successori
		
		
		for(int i = 0; i < n ; i ++){
			IntDomainVar[] succ = successori(v,i,grafo);
			if(succ!=null){
				p.post(p.implies(p.eq(p.sum(succ),0),p.eq(v[i],0)));
			}
		}
		
		//predecessori
		
		
		for(int i = 0; i < n ; i ++){
			IntDomainVar[] pred = predecessori(v,i,grafo);
			if(pred!=null){
				p.post(p.implies(p.eq(p.sum(pred),0),p.eq(v[i],0)));
			}
			
		}
			
		for(int i = archiIniziali.length; i < n ; i ++){
			IntDomainVar[] pred = predecessori(v,i,grafo);
			if(pred!=null){
				p.post(p.implies(p.lt(0,v[i]),p.eq(p.sum(pred),p.minus(v[i],1))));
			}else{
				try{
					IntDomainVar[] succ = successori(v,i,grafo);
					v[i].setVal(0);
					if(succ!=null)
						p.post(p.eq(p.sum(succ),0));
				}catch(Exception ex){
				}
			}
		}	
		
		/*******************************************************
		 *Funzione obiettivo
		 */
		 	
		p.minimize(costo,false);
		
		Solver s = p.getSolver();
    	p.solve(true);
    	
    	int soluzioniTrovate = s.getSearchSolver().solutions.size();
    	
    	if(soluzioniTrovate>0){
    		String[] soluzioni = new String[n+1];
    		Solution solution = (Solution) s.getSearchSolver().solutions.get(0);
     			
 			for (int j = 0; j < n+1; j++) {
     			soluzioni[j] = ""+solution.getValue(j);
 			}
 			return soluzioni;
    	}else{
    		return null;
    	}
	}
	
	/*****************************************************************/
	// Funzione che preso in ingresso un indice i, mi restituisce i successori 
	//dell'arco (i,j) nella tabella grafo
	public static IntDomainVar[] successori(IntDomainVar[] v,int i, int[][] grafo){
		IntDomainVar[] succ;
		int num = 0;
		for(int j = 0; j < grafo.length ;j++){
			if(grafo[i][1] == grafo[j][0]){
				num++;
			}
		}
		
		if(num>0){
			succ = new IntDomainVar[num];
			int k = 0;
			for(int j = 0; j < grafo.length ;j++){
				if(grafo[i][1] == grafo[j][0]){
					succ[k] = v[j];
					k++;
				}
			}
			return succ;
		}
		else{
			return null;
		}
	}
	/*****************************************************************/
	// Funzione che preso in ingresso un indice i, mi restituisce i predecessori 
	//dell'arco (i,j) nella tabella grafo
	public static IntDomainVar[] predecessori(IntDomainVar[] v,int i, int[][] grafo){
		IntDomainVar[] pred;
		int num = 0;
		for(int j = 0; j < grafo.length ;j++){
			if(grafo[i][0] == grafo[j][1]){
				num++;
			}
		}
		
		if(num>0){
			pred = new IntDomainVar[num];
			int k = 0;
			for(int j = 0; j < grafo.length ;j++){
				if(grafo[i][0] == grafo[j][1]){
					pred[k] = v[j];
					k++;
				}
			}
			return pred;
		}
		else{
			return null;
		}
	}
	public static boolean presente(IntDomainVar[] lista, IntDomainVar v){
		for(int i = 0; i < lista.length; i++){
			if(lista[i].equals(v)){
				return true;
			}
		}
		return false;
	}
	public static IntDomainVar[] archiIniziali(IntDomainVar[] v,int[][] grafo){
		int n = 0;
		for(int i = 0; i < grafo.length; i++){
			if(grafo[i][0] == 0)
				n++;
		}
		
		IntDomainVar[] archiIniziali = new IntDomainVar[n];
		for(int i = 0; i < n; i++){
			archiIniziali[i] = v[i];
		}
		
		return archiIniziali;
	}
	public static IntDomainVar[] archiFinale(IntDomainVar[] v,int[][] grafo,boolean[][] m){
		int n = 0;
		for(int i = 0; i < grafo.length; i++){
			if(grafo[i][1] == m.length-1)
				n++;
		}
		
		IntDomainVar[] archiFinali = new IntDomainVar[n];
		int j = 0;
		for(int i = 0; i < grafo.length; i++){
			if(grafo[i][1] == m.length-1){
				archiFinali[j] = v[i];
				j++;
			}
		}
		
		return archiFinali;
	}
	public static int[] durateArchi(int[] durate,int[][] grafo){
		int[] durateArchi = new int[grafo.length];
		
		for(int i = 0; i < grafo.length ; i++){
			for(int j = 0; j < grafo.length; j++){
				if(grafo[j][0] == i){
					durateArchi[j] = durate[i];
				}
			}
		}
		return durateArchi;
	}
	public static int[] costoArchi(int[] costi,int[][] grafo){
		int[] costoArchi = new int[grafo.length];
		
		for(int i = 0; i < grafo.length ; i++){
			for(int j = 0; j < grafo.length; j++){
				if(grafo[j][0] == i){
					costoArchi[j] = costi[i];
				}
			}
		}
		return costoArchi;
	}
	public static int[][] tabellaGrafo(boolean[][] matriceAd){
		int n = contaTrue(matriceAd);
		if(n!=0){
			int[][] grafo = new int[n][2];
			int k = 0;
			int f = grafo.length-1;
			for(int i = 0; i < matriceAd.length; i++){
				for(int j = 0; j < matriceAd[0].length; j++){
					if(matriceAd[i][j]){
						if(j == matriceAd.length-1){
							grafo[f][0] = i;
							grafo[f][1] = j;
							f--;
						}else{
							grafo[k][0] = i;
							grafo[k][1] = j;
							k++;
						}
					}
				}
			}
			return grafo;
		}else{
			return null;
		}
	}
	
	private static int contaTrue(boolean[][] m){
		int n = 0;
		
		for(int i = 0; i < m.length; i++){
			for(int j = 0; j < m[0].length; j++){
				if(m[i][j])
					n++;
			}
		}
		return n;
	}
	public static void main(String[] args){
		boolean[][] m = new boolean[5][5];
		for(int i = 0; i < 5; i++){
			for(int j = 0; j < 5; j++){
				m[i][j] = false;
			}
		}
		
		m[0][2] = true;
		m[1][2] = true;
		m[1][4] = true;
		m[2][3] = true;
		m[3][1] = true;
		m[3][4] = true;
	}
}