/*************************************************************************/
/********* Funzionamento algoritmo ***************************************/
//  Ad ogni attivita � assegnato un valore intero compreso tra 0
//	e il numero delle attivita meno uno :  0<= attivita <=num-1	
//	
//	passo 1 : data la matrice di adiacenza vengono determinati
//	i livelli per poter disegnare i nodi uno dopo l'altro: livelli[]
//
//	passo 2 : calcolati i livelli vengono creati n nodi, dove ogni
//	nodo[i] rappresenta l'attivita livelli[i], creati in base ai livelli
//
//  passo 3 : assegnazioneLivelli: vengno assegnati le coordinate dove
//	i nodi verrano poi disegnati in base alle precedenze
//
//	passo 4 : vengono creati tanti oggetti arco quanti archi ci sono
//	nel grafo da disegnare : serve per disegnare il cammino critico
//
//	passo 5 : disegno nodi e archi orientati. 
//
/*************************************************************************/


import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

class Nodo{
	private int attivita;
	private String nome;
	private int durata;
	private int x;
	private int y;
	public Nodo(int attivita,String nome,int dur){
		this.x = 50;
		this.y = 50;
		this.attivita = attivita;
		this.nome = nome;
		this.durata = dur;
	}
	public void prossimoLivelloX(){
		this.x += 100;
	}
	public void prossimoLivelloY(){
		this.y += 150;
	}
	public int getAttivita(){
		return attivita;
	}
	public String getNome(){
		return nome;
	}
	public void setX(int x){
		this.x += x*100;
	}
	public void setY(int y){
		this.y = y;
	}
	public int getX(){
		return x;
	}
	public int getY(){
		return y;
	}
	public void disegna(Graphics g){
		g.setColor(Color.black);
		g.fillOval(x,y,40,40);
		
		g.setColor(Color.green);
		g.drawString(durata+"",x+20,y-10);
		
		g.setColor(Color.red);
		g.drawString(nome,x+20,y+20);
	}
}
class Arco{
	private Nodo n1;
	private Nodo n2;
	private int durata;
	public Arco(Nodo n, Nodo m){
		n1 = n;
		n2 = m;
	}
	public Arco(Nodo n, Nodo m,int d){
		n1 = n;
		n2 = m;
		durata = d;
	}
	public void disegna(Graphics g){
		g.setColor(Color.green);
		g.drawLine(n1.getX()+20,n1.getY()+20,n2.getX(),n2.getY()+20);
		g.setColor(Color.green);
		g.fillOval(n2.getX()-5,n2.getY()+15,10,10);
		
		
		//g.setColor(Color.green);
		//g.drawString(durata+"",(n1.getX()+n2.getX())/2+5,(n1.getY()+n2.getY())/2+5);
	}
	public void disegnaConDurata(Graphics g){
		g.setColor(Color.green);
		g.drawLine(n1.getX()+20,n1.getY()+20,n2.getX(),n2.getY()+20);
		g.setColor(Color.green);
		g.fillOval(n2.getX()-5,n2.getY()+15,10,10);
		
		g.setColor(Color.green);
		g.drawString(durata+"",(n1.getX()+n2.getX())/2+5,(n1.getY()+n2.getY())/2+5);
	}
}

public class PanelDisegnoPert extends JComponent{
	//matrice di adiacenza
	boolean[][] grafo;
	int num;
	
	//gestioni nodi
	Nodo[] nodi;
	
	//gestione archi
	Arco[] archi;
	
	//gestione livelli 
	//righe = livelli disegno
	//colonne = attivita
	int[] livelli;
	
	//scroll
	JScrollPane scroll;
	
	Graphics2D g;
	
	public PanelDisegnoPert(boolean[][] g,String[] att, int[] dur){
		
		//assegno grafo e precedenze
		grafo = g;
		num = g[0].length;
		
		//gestione livelli
		livelli = new int[num];
		initLivelli();
		precedenza(livelli);
		
		//gestione nodi
		creaNodi(att,dur);
		
		//gestione archi
		creaArchi(g,dur);
		
		assegnaLivelli();
		
		creaScroll();
		
		this.setPreferredSize(new Dimension(10000, 2000));
	}  
	
	public void creaScroll(){
		scroll = new JScrollPane();
		scroll.setAutoscrolls(true);
		scroll.setLocation(0,0);
		scroll.setSize(500,500);
		scroll.getViewport().add(this);
	}
	
	public JScrollPane getScroll(){
		return scroll;
	}
	
	public void creaNodi(String[] att,int[] dur){
		nodi = new Nodo[num];
		for(int i = 0; i < num; i++){

			nodi[livelli[i]] = new Nodo(livelli[i],att[livelli[i]],dur[i]);
			nodi[livelli[i]].setX(i);
		}		
	}
	public void creaArchi(boolean[][] g,int[] dur){
		archi = new Arco[contaArchi(g)];
		int k = 0;
		for(int i = 0; i < num; i++){
			for(int j = 0; j < num; j++){
				if(presenteArco(nodi[i].getAttivita(),nodi[j].getAttivita())){
					archi[k] = new Arco(nodi[i],nodi[j],dur[i]);
					k++;
				}
			}
		}
	}
	public int contaArchi(boolean[][] g){
		int tmp = 0;
		for(int i = 0; i < num; i++){
			for(int j = 0; j < num; j++){
				if(g[i][j])
					tmp++;
			}
		}
		return tmp;
	}
	public void paintComponent(Graphics gg){
		super.paintComponent(gg);
		g = (Graphics2D) gg;
		
		disegna(g);
	    
	}
	
	public void disegna(Graphics2D g){
		//disegno tutti gli archi
		for(int i = 0; i < archi.length; i++){
			archi[i].disegna(g);
		}
		
		//disegno tutte le attivita
		for(int i = 0; i < num; i++){
			nodi[i].disegna(g);
		}
	}
	
	public void assegnaLivelli(){
		for(int k = 0; k < num; k++){
			for(int i = 0; i < num; i++){
				if(i<=k){
					for(int j = 0; j < num; j++){
						if(j>=k){
							if(presenteArco(i,j)){
								
								if(i != k && k != j && nodi[i].getY()==nodi[k].getY() /*&& nodi[livelli[k]].getY()==nodi[livelli[j]].getY()*/){
									nodi[k].prossimoLivelloY();
								}
							}
						}
					}
				}
			}
		}
	}
	
	public boolean presenteArco(int a, int b){
		if(grafo[a][b])
			return true;
		else
			return false;
	}
	
	public boolean verificaPrecedenza(int a, int b){
		if(grafo[a][b])
			return true;
		else
			return false;
	}
	
	public void precedenza(int[] livelli){
		
		for(int i = 0; i < num; i++){
			for(int j = 0; j < num; j++){
				if(i!=j){
					if(verificaPrecedenza(livelli[i],livelli[j])){
						if(i>j){
							int tmp = livelli[i];
							livelli[i] = livelli[j];
							livelli[j] = tmp;
						}
					}
				}
			}
		}
	}
	public void initLivelli(){
		int n = num-1;
		for(int i = 0; i < num; i++){
			livelli[i] = n;
			n--;
			
		}
	}
	public int[] getLivelli(){
		return livelli;
	}
	public void aggiorna(boolean[][] mg,String[] att, int[] dur){
 		//assegno grafo e precedenze
		grafo = mg;
		num = mg[0].length;
		
		//gestione livelli
		livelli = new int[num];
		initLivelli();
		precedenza(livelli);
		
		//gestione nodi
		creaNodi(att,dur);
		
		//gestione archi
		creaArchi(mg,dur);
		
		assegnaLivelli();
		
 		repaint();
 	}
}